/**
 * js used on single wishlist page
 */

/**
 * Common js code
 */

var nmgr = window.nmgr || {};

jQuery(function($) {

    /**
     * Define a common blockUI block setup for plugin
     *
     * @param {string | jQuery object} selector The element to be blocked
     */
    nmgr.block = function(selector) {
        if (!nmgr.is_blocked(selector)) {
            $(selector).addClass('is-blocked').block({
                message: null,
                overlayCSS: {
                    backgroundColor: '#fff'
                },
                css: {
                    backgroundColor: 'transparent',
                    border: 'none'
                }
            });
        }
    };

    /**
     * Define a common blockUI unblock setup for plugin
     *
     * @param {string | jQuery object} selector The element to be unblocked
     */
    nmgr.unblock = function(selector) {
        $(selector).removeClass('is-blocked').unblock();
    };

    /**
     * Check whether an element is currently blocked with blockUI
     *
     * @param {string | jQuery object} selector The element to be checked
     * @returns {jQuery.length|Boolean} True if the element is blocked, false if not
     */
    nmgr.is_blocked = function(selector) {
        return $(selector).is('is-blocked') || $(selector).parents('is-blocked').length;
    };

    /**
     * Scroll to an element
     *
     * @param {string | jQuery object} selector The element to scroll to
     */
    nmgr.scroll_to = function(selector) {
        if ($(selector).length) {
            $('html, body').animate({
                scrollTop: ($(selector).offset().top - 100)
            }, 1000);
        }
    };

    /**
     * Go to a url
     * @param {string} url
     */
    nmgr.go_to = function(url) {
        if (-1 === url.indexOf('https://') || -1 === url.indexOf('http://')) {
            window.location = url;
        } else {
            window.location = decodeURI(url);
        }
    };

    /**
     * Remove woocommerce notices from the page
     * @returns {undefined}
     */
    nmgr.remove_notice = function() {
        $('.woocommerce-error, .woocommerce-message, .woocommerce-info').remove();
    };

    /**
     * Make a table responsive when it's width is above a certain size.
     *
     * The responsiveness in this case is for desktop display rather than mobile.
     * By default, the table should already be mobile responsive.
     *
     * @param {int} width Minimum width at which table would be responsive. Default 700px
     */
    nmgr.make_table_responsive = function() {
        var table = $('.nmgr-table');
        if (table.width() > 700) {
            table.addClass('responsive');
        } else {
            table.removeClass('responsive');
        }
    };

    nmgr.make_table_responsive();
    $(window).on('resize', nmgr.make_table_responsive);

    nmgr.tiptip = function() {
        $('.nmgr-tip').tipTip();

        $('svg.nmgr-tip').each(function() {
            var $element = $(this),
                title = $element.find('title');

            if (!title.length) {
                return;
            }

            $element.tipTip({
                content: function() {
                    if (title.length) {
                        if (title.attr('data-title')) {
                            title.text('');
                            return title.attr('data-title');
                        }
                    }
                }
            });
        });
    };

    nmgr.tiptip();


});


jQuery(function($) {
    var stupidtable = {
        init: function() {
            $('.nmgr-items-table').stupidtable();
            $('.nmgr-items-table').on('aftertablesort', this.add_arrows);
        },

        add_arrows: function(event, data) {
            var th = $(this).find('th');
            var arrow = data.direction === 'asc' ? '&uarr;' : '&darr;';
            var index = data.column;
            th.find('.nmgr-arrow').remove();
            th.eq(index).append('<span class="nmgr-arrow">' + arrow + '</span>');
        }
    };

    var make_responsive = function() {
        var table = $('.nmgr-items-table');
        if (table.width() > 700) {
            table.addClass('responsive');
        } else {
            table.removeClass('responsive');
        }
    };

    var add_to_cart = {
        init: function() {
            if (typeof nmgr_wishlist_params === 'undefined') {
                return false;
            }

            $(document.body)
                .on('click', '.nmgr_add_to_cart_button', this.add)
                .on('nmgr_added_to_cart', this.refresh_cart_fragments);
        },

        show_notice: function(html) {
            $target = $('.woocommerce-notices-wrapper:first') || $('.nmgr_add_to_wishlist_button').closest('.woocommerce');

            if ($target.length) {
                nmgr.remove_notice();
                $target.prepend(html);
                $('html, body').animate({
                    scrollTop: ($target.offset().top - 100)
                }, 1000);
            }
        },

        add: function(e) {
            var $thisbutton = $(this),
                $form = $thisbutton.closest('.nmgr-add-to-cart-form');
            /**
             *  If we are not adding to cart via ajax, we are supposed to return here
             *  but we will only return if adding the wishlist item to the cart is not restricted.
             *  Otherwise we will continue into the ajax code below in order to retrieve the
             *  restricted notice set during the add-to-cart validation
             */
            if (!$thisbutton.is('.nmgr_ajax_add_to_cart')) {
                return true;
            }

            e.preventDefault();

            var data = {
                action: 'nmgr_add_to_cart',
                security: nmgr_wishlist_params.nonce
            };

            var formdata = $form.serializeArray();

            $(formdata).each(function(index, obj) {
                data[obj.name] = obj.value;
            });

            $thisbutton.removeClass('added');
            $thisbutton.addClass('loading');

            $.post(nmgr_wishlist_params.ajax_url, data, function(response) {
                if (!response) {
                    return;
                }

                $('.woocommerce-error, .woocommerce-message, .woocommerce-info').remove();
                $thisbutton.removeClass('loading');

                if (response.notice) {
                    add_to_cart.show_notice(response.notice);
                }

                if (response.success) {
                    $(document.body).trigger('nmgr_added_to_cart', [$form]);
                }
            });
        },

        refresh_cart_fragments: function() {
            var data = {
                action: 'nmgr_get_cart_fragments',
                security: nmgr_wishlist_params.nonce
            };
            $.post(nmgr_wishlist_params.ajax_url, data, function(response) {
                if (!response) {
                    return;
                }

                if (response.fragments) {
                    $.each(response.fragments, function(key) {
                        $(key)
                            .addClass('updating')
                            .fadeTo('400', '0.6')
                            .block({
                                message: null,
                                overlayCSS: {
                                    opacity: 0.6
                                }
                            });
                    });

                    $.each(response.fragments, function(key, value) {
                        $(key).replaceWith(value);
                        $(key).stop(true).css('opacity', '1').unblock();
                    });

                    $(document.body).trigger('wc_fragments_loaded');
                }
            });
        }

    };

    stupidtable.init();
    make_responsive();
    $(window).on('resize', make_responsive);
    add_to_cart.init();
});