<?php

defined( 'ABSPATH' ) || exit;

class NMGR_Scripts {

	/**
	 * Suffix to append to file name before file extension e.g. 'min' for minified
	 *
	 * @var string
	 */
	private static $suffix;

	/**
	 * Localized script handles
	 *
	 * @var array
	 */
	private static $localized_scripts = array();

	public static function run() {
		self::$suffix = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '.min';

		// 99 priority ensures we hook our scripts after woocommerce's
		add_action( 'admin_enqueue_scripts', array( __CLASS__, 'admin_scripts' ), 99 );
		add_action( 'wp_enqueue_scripts', array( __CLASS__, 'frontend_scripts' ), 99 );

		add_action( 'wp_print_scripts', array( __CLASS__, 'localize_printed_scripts' ), 5 );
		add_action( 'wp_print_footer_scripts', array( __CLASS__, 'localize_printed_scripts' ), 5 );

		add_action( 'wp_footer', 'nmgr_include_sprite_file' );
		add_action( 'admin_footer', 'nmgr_include_sprite_file' );
	}

	public static function frontend_scripts() {
		self::register_common_scripts();
		self::register_frontend_scripts();

		wp_enqueue_style( 'nmgr-add-to-wishlist' );
		wp_enqueue_script( 'nmgr-add-to-wishlist' );

		if ( is_nmgr_wishlist() ) {
			wp_enqueue_style( 'nmgr-wishlist' );
			wp_enqueue_script( 'nmgr-wishlist' );
		}

		if ( is_nmgr_search() ) {
			wp_enqueue_style( 'nmgr-search' );
		}

		if ( is_nmgr_account() || is_nmgr_account_section() ) {
			wp_enqueue_style( 'nmgr-account' );
			wp_enqueue_script( 'nmgr-account' );
		}

		wp_add_inline_script( 'nmgr-account', self::get_account_inline_script() );
	}

	public static function admin_scripts() {
		self::register_common_scripts();
		self::register_admin_scripts();

		if ( is_nmgr_admin() ) {
			wp_enqueue_style( 'nmgr-admin' );
			wp_enqueue_script( 'nmgr-admin' );
		}
	}

	/**
	 * Register scripts used on both admin and frontend
	 *
	 * Because woocommerce also registers some of these scripts, we have to check
	 * if the script it registered so that we can use woocommerce's own and not override it
	 * with our own or add our script to the page if it already exists
	 */
	private static function register_common_scripts() {
		/**
		 * Styles
		 */
		wp_register_style( 'nmgr-jquery-tiptip', nmgr()->url . 'assets/css/vendor/jquery-tiptip' . self::$suffix . '.css', array() );

		if ( !wp_style_is( 'select2', 'registered' ) ) {
			wp_register_style( 'select2', nmgr()->url . 'assets/css/vendor/select2.css', array() );
		}

		/**
		 * Scripts
		 */
		if ( !wp_script_is( 'stupidtable', 'registered' ) ) {
			wp_register_script( 'stupidtable', nmgr()->url . 'assets/js/vendor/stupidtable' . self::$suffix . '.js', array( 'jquery' ), '', true );
		}

		if ( !wp_script_is( 'jquery-blockui', 'registered' ) ) {
			wp_register_script( 'jquery-blockui', nmgr()->url . 'assets/js/vendor/jquery.blockUI' . self::$suffix . '.js', array( 'jquery' ), '2.70', true );
		}

		if ( !wp_script_is( 'jquery-tiptip', 'registered' ) ) {
			wp_register_script( 'jquery-tiptip', nmgr()->url . 'assets/js/vendor/jquery.tipTip' . self::$suffix . '.js', array( 'jquery' ), nmgr()->version, true );
		}

		// We're forced to use selectWoo instead of select2 because woocommerce uses it for shipping fields
		if ( !wp_script_is( 'selectWoo', 'registered' ) ) {
			wp_register_script( 'selectWoo', nmgr()->url . 'assets/js/vendor/selectWoo.full' . self::$suffix . '.js', array( 'jquery' ), '1.0.6', true );
		}

		wp_register_script( 'nmgr-pikaday', nmgr()->url . 'assets/js/vendor/pikaday' . self::$suffix . '.js', array(), '1.8.0', true );
	}

	private static function register_admin_scripts() {
		/**
		 * Styles
		 */
		wp_register_style( 'nmgr-admin', nmgr()->url . 'assets/css/admin' . self::$suffix . '.css', array( 'nmgr-jquery-tiptip' ), nmgr()->version );

		/**
		 * Scripts
		 */
		wp_register_script( 'nmgr-admin-bootstrap', nmgr()->url . 'assets/js/vendor/bootstrap-native' . self::$suffix . '.js', array(), nmgr()->version, true );

		wp_register_script( 'nmgr-admin', nmgr()->url . 'assets/js/admin' . self::$suffix . '.js', array( 'jquery', 'selectWoo', 'stupidtable', 'jquery-blockui', 'nmgr-admin-bootstrap', 'nmgr-pikaday', 'jquery-tiptip' ), nmgr()->version, true );
	}

	private static function register_frontend_scripts() {
		/**
		 * Styles
		 */
		wp_register_style( 'nmgr-add-to-wishlist', nmgr()->url . 'assets/css/add-to-wishlist' . self::$suffix . '.css', array( 'nmgr-jquery-tiptip' ), nmgr()->version );

		wp_register_style( 'nmgr-account', nmgr()->url . 'assets/css/account' . self::$suffix . '.css', array( 'nmgr-jquery-tiptip', 'select2' ), nmgr()->version );

		wp_register_style( 'nmgr-search', nmgr()->url . 'assets/css/search' . self::$suffix . '.css', array(), nmgr()->version );

		wp_register_style( 'nmgr-wishlist', nmgr()->url . 'assets/css/wishlist' . self::$suffix . '.css', array( 'nmgr-jquery-tiptip' ), nmgr()->version );

		/**
		 * Scripts
		 */
		wp_register_script( 'nmgr-add-to-wishlist', nmgr()->url . 'assets/js/add-to-wishlist' . self::$suffix . '.js', array( 'jquery', 'jquery-tiptip', 'wc-add-to-cart-variation' ), nmgr()->version, true );

		wp_register_script( 'nmgr-account', nmgr()->url . 'assets/js/account' . self::$suffix . '.js', array( 'jquery', 'stupidtable', 'jquery-blockui', 'jquery-tiptip', 'wc-country-select', 'wc-address-i18n', 'nmgr-pikaday', 'selectWoo' ), nmgr()->version, true );

		wp_register_script( 'nmgr-wishlist', nmgr()->url . 'assets/js/wishlist' . self::$suffix . '.js', array( 'jquery', 'stupidtable', 'jquery-tiptip' ), nmgr()->version, true );

		wp_register_script( 'nmgr-enable-wishlist', nmgr()->url . 'assets/js/enable-wishlist' . self::$suffix . '.js', array(), nmgr()->version, true );
	}

	private static function get_script_data( $handle ) {
		$data = array();
		$nmgr_global = isset( $GLOBALS[ 'nmgr' ] ) ? $GLOBALS[ 'nmgr' ] : '';
		$ajax_url = admin_url( 'admin-ajax.php' );
		$i18n_pikaday = array(
			'previousMonth' => esc_attr__( 'Previous Month', 'nm-gift-registry-lite' ),
			'nextMonth' => esc_attr__( 'Next Month', 'nm-gift-registry-lite' ),
			'months' => array(
				esc_attr__( 'January', 'nm-gift-registry-lite' ),
				esc_attr__( 'February', 'nm-gift-registry-lite' ),
				esc_attr__( 'March', 'nm-gift-registry-lite' ),
				esc_attr__( 'April', 'nm-gift-registry-lite' ),
				esc_attr__( 'May', 'nm-gift-registry-lite' ),
				esc_attr__( 'June', 'nm-gift-registry-lite' ),
				esc_attr__( 'July', 'nm-gift-registry-lite' ),
				esc_attr__( 'August', 'nm-gift-registry-lite' ),
				esc_attr__( 'September', 'nm-gift-registry-lite' ),
				esc_attr__( 'October', 'nm-gift-registry-lite' ),
				esc_attr__( 'November', 'nm-gift-registry-lite' ),
				esc_attr__( 'December', 'nm-gift-registry-lite' )
			),
			'weekdays' => array(
				esc_attr__( 'Sunday', 'nm-gift-registry-lite' ),
				esc_attr__( 'Monday', 'nm-gift-registry-lite' ),
				esc_attr__( 'Tuesday', 'nm-gift-registry-lite' ),
				esc_attr__( 'Wednesday', 'nm-gift-registry-lite' ),
				esc_attr__( 'Thursday', 'nm-gift-registry-lite' ),
				esc_attr__( 'Friday', 'nm-gift-registry-lite' ),
				esc_attr__( 'Saturday', 'nm-gift-registry-lite' )
			),
			'weekdaysShort' => array(
				_x( 'Sun', 'weekday', 'nm-gift-registry-lite' ),
				_x( 'Mon', 'weekday', 'nm-gift-registry-lite' ),
				_x( 'Tue', 'weekday', 'nm-gift-registry-lite' ),
				_x( 'Wed', 'weekday', 'nm-gift-registry-lite' ),
				_x( 'Thu', 'weekday', 'nm-gift-registry-lite' ),
				_x( 'Fri', 'weekday', 'nm-gift-registry-lite' ),
				_x( 'Sat', 'weekday', 'nm-gift-registry-lite' )
			)
		);

		// Parameters that can be used by various scripts
		$global_params = array(
			'global' => $nmgr_global,
			'ajax_url' => $ajax_url,
			'i18n_pikaday' => $i18n_pikaday,
			'i18n_delete_item_text' => sprintf(
				/* translators: %s: wishlist type title */
				esc_attr__( 'Are you sure you want to remove the %s item?', 'nm-gift-registry-lite' ), esc_html( nmgr_get_type_title() ) ),
			'items_nonce' => wp_create_nonce( 'nmgr-items' ),
		);


		$data[ 'nmgr-account' ] = array(
			'global' => $global_params,
			'overview_nonce' => wp_create_nonce( 'nmgr-overview' ),
			'profile_nonce' => wp_create_nonce( 'nmgr-profile' ),
			'shipping_nonce' => wp_create_nonce( 'nmgr-shipping' ),
			'require_shipping_address' => nmgr_get_option( 'shipping_address_required' ),
			'i18n_use_account_shipping_address_text' => esc_attr__( 'Use your account shipping address? This will remove any currently entered shipping information.', 'nm-gift-registry-lite' ),
			'i18n_copied_text' => esc_attr__( 'Copied', 'nm-gift-registry-lite' ),
		);

		$data[ 'nmgr-add-to-wishlist' ] = array(
			'ajax_url' => $ajax_url,
			'nonce' => wp_create_nonce( 'nmgr-add-to-wishlist' ),
			'i18n_select_product_text' => sprintf(
				/* translators: %s: wishlist type title */
				esc_attr__( 'Please select an item from the dropdown to add to your %s.', 'nm-gift-registry-lite' ), esc_html( nmgr_get_type_title() ) ),
			'i18n_select_quantity_text' => sprintf(
				/* translators: %s: wishlist type title */
				esc_attr__( 'Please choose the quantity of items you wish to add to your %s from the dropdown.', 'nm-gift-registry-lite' ), esc_html( nmgr_get_type_title() ) ),
			'i18n_make_a_selection_text' => sprintf(
				/* translators: %s wishlist type title */
				esc_attr__( 'Please select some product options before adding this product to your %s.', 'nm-gift-registry-lite' ),
				esc_html( nmgr_get_type_title() ) ),
			'i18n_unavailable_text' => esc_attr__( 'Sorry, this product is unavailable. Please choose a different combination.', 'nm-gift-registry-lite' ),
		);

		$data[ 'nmgr-wishlist' ] = array(
			'ajax_url' => $ajax_url,
			'nonce' => wp_create_nonce( 'nmgr-wishlist' ),
		);

		$disable_notice_single = sprintf(
			/* translators: 1,2: wishlist type title */
			esc_attr__( 'Are you sure you want to disable the %1$s module? This would delete your %2$s and all its data.', 'nm-gift-registry-lite' ),
			esc_html( nmgr_get_type_title() ),
			esc_html( nmgr_get_type_title() )
		);

		$data[ 'nmgr-enable-wishlist' ] = array(
			'disable_notice' => $disable_notice_single
		);

		$data[ 'nmgr-admin' ] = array(
			'global' => $global_params,
			'i18n_use_account_shipping_address_text' => esc_attr__( "Use the user's account shipping address? This will remove any currently entered shipping information.", 'nm-gift-registry-lite' ),
			'i18n_select_user' => esc_attr__( 'You have not selected a user yet. Please select a user', 'nm-gift-registry-lite' ),
			'search_users_nonce' => wp_create_nonce( 'nmgr-search-users' ),
			'search_products_nonce' => wp_create_nonce( 'nmgr-search-products' ),
			'countries' => wp_json_encode( array_merge( WC()->countries->get_allowed_country_states(), WC()->countries->get_shipping_country_states() ) ),
			'i18n_select_state_text' => esc_attr__( 'Select an option&hellip;', 'nm-gift-registry-lite' ),
		);

		return isset( $data[ $handle ] ) ? $data[ $handle ] : false;
	}

	private static function localize_script( $handle ) {
		if ( !in_array( $handle, self::$localized_scripts, true ) && wp_script_is( $handle ) ) {
			$data = self::get_script_data( $handle );

			if ( !$data ) {
				return;
			}

			// Has this script been localized
			$localized = false;

			if ( isset( $data[ 'global' ] ) ) {
				wp_localize_script( $handle, 'nmgr_global_params', $data[ 'global' ] );
				$localized = true;
				unset( $data[ 'global' ] );
			}

			if ( !empty( $data ) ) {
				$name = str_replace( '-', '_', $handle ) . '_params';
				wp_localize_script( $handle, $name, $data );
				$localized = true;
			}

			if ( $localized ) {
				self::$localized_scripts[] = $handle;
			}
		}
	}

	/**
	 * Localize all nmgr enqueued scripts if necessary
	 */
	public static function localize_printed_scripts() {
		global $wp_scripts;
		foreach ( $wp_scripts->queue as $handle ) {
			if ( false !== strpos( $handle, 'nmgr-' ) ) {
				self::localize_script( $handle );
			}
		}
	}

	/**
	 * Inline script used for account sections
	 *
	 * @return string
	 */
	public static function get_account_inline_script() {
		$script = '';
		$notice = wc()->session->get( 'nmgr_notice' );

		// Hard code all these for now
		if ( 'require-shipping-address' === $notice ) {
			wc()->session->set( 'nmgr_notice', null );
			$script = "
				jQuery(function ($) {
					nmgr.account_tabs.showContent('#nmgr-tab-shipping');
				});
			";
		}
		return $script;
	}

}
